@@ -4,6 +4,10 @@ class RDDLActionPreconditionNotSatisfiedError(ValueError):
     pass
 
 
+class RDDLInvalidActionError(ValueError):
+    pass
+
+
 class RDDLInvalidDependencyInCPFError(SyntaxError):
     pass
 
@@ -16,6 +20,10 @@ class RDDLInvalidNumberOfArgumentsError(SyntaxError):
     pass
 
 
+class RDDLInvalidObjectError(SyntaxError):
+    pass
+
+
 class RDDLMissingCPFDefinitionError(SyntaxError):
     pass
 
@@ -28,6 +36,10 @@ class RDDLParseError(SyntaxError):
     pass
 
 
+class RDDLRepeatedVariableError(SyntaxError):
+    pass
+    
+
 class RDDLStateInvariantNotSatisfiedError(ValueError):
     pass
 
@@ -53,4 +65,4 @@ class RDDLEnvironmentNotExist(ValueError):
 
 
 class RDDLInstanceNotExist(ValueError):
-    pass
\ No newline at end of file
+    pass
