@@ -3,41 +3,34 @@ import copy
 import itertools
 import warnings
 
-from pyRDDLGym.Core.Grounder.RDDLModel import RDDLModel
 from pyRDDLGym.Core.ErrorHandling.RDDLException import RDDLInvalidExpressionError
 from pyRDDLGym.Core.ErrorHandling.RDDLException import RDDLInvalidNumberOfArgumentsError
 from pyRDDLGym.Core.ErrorHandling.RDDLException import RDDLMissingCPFDefinitionError
+from pyRDDLGym.Core.ErrorHandling.RDDLException import RDDLRepeatedVariableError
+from pyRDDLGym.Core.ErrorHandling.RDDLException import RDDLTypeError
 from pyRDDLGym.Core.ErrorHandling.RDDLException import RDDLUndefinedVariableError
 from pyRDDLGym.Core.ErrorHandling.RDDLException import RDDLValueOutOfRangeError
+
+from pyRDDLGym.Core.Compiler.RDDLModel import RDDLGroundedModel
 from pyRDDLGym.Core.Parser.expr import Expression
 
-# import RDDLModel
-
-# AGGREG_OPERATION_STRING_LIST = ["sum","prod","max","min","avg"]
-# AGGREG_RECURSIVE_OPERATION_STRING_MAPPED_LIST = ["+","*","max","min","+"]
-# QUANTIFIER_OPERATION_STRING_LIST = ["forall","exists"]
-# QUANTIFIER_RECURSIVE_OPERATION_STRING_MAPPED_LIST = ["&","|"]
-
 PRIME = '\''
 
-AGGREG_OPERATION_LIST = [
-    'prod', 'sum', 'avg', 'minimum', 'maximum', 'forall', 'exists'
-]
-AGGREG_RECURSIVE_OPERATION_INDEX_MAPPED_LIST = [
-    '*', '+', '+', 'min', 'max', '^', '|'
-]
-AGGREG_OP_TO_STRING_DICT = dict(
-    zip(AGGREG_OPERATION_LIST,
-        AGGREG_RECURSIVE_OPERATION_INDEX_MAPPED_LIST)
-)
-
-
+AGGREG_OP_TO_STRING_DICT = {
+    'prod': '*',
+    'sum': '+',
+    'avg': '+',
+    'minimum': 'min',
+    'maximum': 'max',
+    'forall': '^',
+    'exists': '|'
+}
 
 
 class Grounder(metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
-    def Ground(self) -> RDDLModel:
+    def Ground(self) -> RDDLGroundedModel:
         pass
 
 
@@ -48,6 +41,10 @@ class RDDLGrounder(Grounder):
         self.AST = RDDL_AST
         self.objects = {}
         self.objects_rev = {}
+        self.pvar_to_type = {pvar.name: pvar.range 
+                             for pvar in self.AST.domain.pvariables}
+        self.gvar_to_type = {}
+        self.gvar_to_pvar = {}
         self.nonfluents = {}
         self.states = {}
         self.statesranges = {}
@@ -59,6 +56,7 @@ class RDDLGrounder(Grounder):
         self.actionsranges = {}
         self.cpfs = {}
         self.cpforder = {0: []}
+        self.gvar_to_cpforder = {}
         self.derived = {}
         self.interm = {}
         self.observ = {}
@@ -67,8 +65,17 @@ class RDDLGrounder(Grounder):
         self.terminals = []
         self.preconditions = []
         self.invariants = []
+        
+        self.index_of_object = {}
+        self.enum_types = set()
+        self.enum_literals = set()
+        
+        self.param_types = {}
+        self.variable_types = {}
+        self.variable_ranges = {}
+        self.grounded_names = {}   
 
-    def Ground(self) -> RDDLModel:
+    def Ground(self) -> RDDLGroundedModel:
         self._extract_objects()
         self._ground_non_fluents()
         self._ground_pvariables_and_cpf()
@@ -76,17 +83,20 @@ class RDDLGrounder(Grounder):
         # self._groundPvariables()
         self.reward = self._scan_expr_tree(self.AST.domain.reward, {})  # empty args dictionary
         self._ground_constraints()
+        
+        self._extract_variable_information()
 
         # update model object
-        model = RDDLModel()
+        model = RDDLGroundedModel()
         model.states = self.states
         model.actions = self.actions
         model.nonfluents = self.nonfluents
         model.next_state = self.nextstates
         model.prev_state = self.prevstates
         model.init_state = self.initstate
-        model.cpfs = self.cpfs
+        model.cpfs = self.cpfs  # changed to mapping name -> (<params>, <expressions>) on Dec 17 (Mike)
         model.cpforder = self.cpforder
+        model.gvar_to_cpforder = self.gvar_to_cpforder
         model.reward = self.reward
         model.terminals = self.terminals
         model.preconditions = self.preconditions
@@ -95,19 +105,35 @@ class RDDLGrounder(Grounder):
         model.interm = self.interm
         model.observ = self.observ
         model.observranges = self.observranges
-        model.objects = self.objects
+        model.objects = {} #self.objects
+        model.objects_rev = {} #self.objects_rev
         model.actionsranges = self.actionsranges
         model.statesranges = self.statesranges
+        model.gvar_to_type = self.gvar_to_type
+        model.pvar_to_type = self.pvar_to_type
+        model.gvar_to_pvar = self.gvar_to_pvar
         model.max_allowed_actions = self._ground_max_actions()
         model.horizon = self._ground_horizon()
         model.discount = self._ground_discount()
+        
+        model.SetAST(self.AST)
+        
+        model.param_types = self.param_types
+        model.variable_types = self.variable_types
+        model.variable_ranges = self.variable_ranges
+        
+        model.index_of_object = self.index_of_object
+        model.enum_types = self.enum_types
+        model.enum_literals = self.enum_literals        
+        model.grounded_names = self.grounded_names
+        
         return model
 
     def _ground_horizon(self):
         horizon = self.AST.instance.horizon
         if not (horizon >= 0):
             raise RDDLValueOutOfRangeError(
-                'Rollout horizon {} in the instance is not >= 0.'.format(horizon))
+                f'Rollout horizon {horizon} in the instance is not >= 0.')
         return horizon
 
     def _ground_max_actions(self):
@@ -121,36 +147,36 @@ class RDDLGrounder(Grounder):
         discount = self.AST.instance.discount
         if not (0. <= discount <= 1.):
             raise RDDLValueOutOfRangeError(
-                'Discount factor {} in the instance is not in [0, 1].'.format(discount))
+                f'Discount factor {discount} in the instance is not in [0, 1].')
         return discount
 
     def _extract_objects(self):
-        if (not self.AST.non_fluents.objects) or self.AST.non_fluents.objects[0] is None:
-            self.objects = None
-            self.objects_rev = None
-        else:
-            self.objects = {}
-            self.objects_rev = {}
+        self.objects = {}
+        self.objects_rev = {}
+        if self.AST.non_fluents.objects \
+        and self.AST.non_fluents.objects[0] is not None:
             for obj_type in self.AST.non_fluents.objects:
                 self.objects[obj_type[0]] = obj_type[1]
                 for obj in obj_type[1]:
                     self.objects_rev[obj] = obj_type[0]
+        self.index_of_object = {}
+        self.enum_types = set()
+        self.enum_literals = set()
 
     def _ground_objects(self, args):
         objects_by_type = []
         for obj_type in args:
             if obj_type not in self.objects:
-                raise RDDLUndefinedVariableError(
-                    'Object type <{}> is not defined: should be one of <{}>.'.format(
-                        obj_type, ','.join(self.objects.keys())))
+                raise RDDLTypeError(
+                    f'Object type <{obj_type}> is not defined, '
+                    f'should be one of <{set(self.objects.keys())}>.')
             objects_by_type.append(self.objects[obj_type])
         return itertools.product(*objects_by_type)
     
     def _append_variation_to_name(self, base_name, variation):
         return base_name + '_' + '_'.join(variation)
         
-    def _generate_grounded_names(self,
-                                 base_name,
+    def _generate_grounded_names(self, base_name,
                                  variation_list,
                                  return_grounding_param_dict=False):
         all_grounded_names = []
@@ -165,26 +191,35 @@ class RDDLGrounder(Grounder):
             grounded_name_to_params_dict[grounded_name] = variation
         if return_grounding_param_dict:
             return all_grounded_names, grounded_name_to_params_dict
-        else:  # in some calls to _generate_name, we do not care about the param dict, hence this
+        else:  
+            # in some calls to _generate_name, we do not care about the param dict
             return all_grounded_names
 
     def _ground_non_fluents(self):
         if not hasattr(self.AST.non_fluents, 'init_non_fluent'):
             return
-        valid_non_fluents = set(
-            pvar.name for pvar in self.AST.domain.pvariables if pvar.is_non_fluent())
+        
+        valid_non_fluents = {pvar.name 
+                             for pvar in self.AST.domain.pvariables 
+                             if pvar.is_non_fluent()}
         for init_vals in self.AST.non_fluents.init_non_fluent:
-            name = init_vals[0][0]
-            if name not in valid_non_fluents:
+            pvar_name = init_vals[0][0]
+            vtype = self.pvar_to_type[pvar_name]
+            if pvar_name not in valid_non_fluents:
                 warnings.warn(
-                    'Non-fluents block initializes an undefined pvariable <{}>.'.format(name),
-                    FutureWarning, stacklevel=2)
+                    f'Non-fluents block initializes undefined variable <{pvar_name}>.',
+                    stacklevel=2)
             variations_list = [init_vals[0][1]]
             val = init_vals[1]
             if variations_list[0] is not None:
                 name = self._generate_grounded_names(
-                    name, variations_list, return_grounding_param_dict=False)[0]    
+                    pvar_name, variations_list, 
+                    return_grounding_param_dict=False)[0]    
+            else:
+                name = pvar_name
             self.nonfluents[name] = val
+            self.gvar_to_type[name] = vtype
+            self.gvar_to_pvar[name] = pvar_name
 
     def _ground_interm(self, pvariable, cpf_set):
         cpf = None
@@ -193,7 +228,7 @@ class RDDLGrounder(Grounder):
                 cpf = cpfs
         if cpf is not None:
             self.derived[pvariable.name] = pvariable.default
-            self.cpfs[pvariable.name] = cpf.expr
+            self.cpfs[pvariable.name] = ([], cpf.expr)
             level = pvariable.level
             if level is None:
                 level = 1
@@ -209,25 +244,33 @@ class RDDLGrounder(Grounder):
         all_grounded_observ_cpfs = []
         for pvariable in self.AST.domain.pvariables:
             name = pvariable.name
+            vtype = self.pvar_to_type[name]
             if pvariable.arity > 0:
-                # In line below, if we leave as an iterator object, will be empty after one iteration. Hence "list(.)"
+                # In line below, if we leave as an iterator object
+                # will be empty after one iteration. Hence "list(.)"
                 variations = list(self._ground_objects(pvariable.param_types))
                 grounded, grounded_name_to_params_dict = self._generate_grounded_names(
-                    name, variations, return_grounding_param_dict=True)
+                    name, variations, 
+                    return_grounding_param_dict=True)
             else:
                 grounded = [name]
                 grounded_name_to_params_dict = {name: []}
             
+            for gvar_name in grounded:
+                self.gvar_to_pvar[gvar_name] = name
+            
             # todo merge martin's code check for abuse of arity                
             if pvariable.fluent_type == 'non-fluent':
                 for g in grounded:
                     if g not in self.nonfluents:
                         self.nonfluents[g] = pvariable.default
+                    self.gvar_to_type[g] = vtype
                 
             elif pvariable.fluent_type == 'action-fluent':
                 for g in grounded:
                     self.actions[g] = pvariable.default
                     self.actionsranges[g] = pvariable.range
+                    self.gvar_to_type[g] = vtype
               
             elif pvariable.fluent_type == 'state-fluent':
                 cpf = None
@@ -238,7 +281,7 @@ class RDDLGrounder(Grounder):
                         break
                 if cpf is None:
                     raise RDDLMissingCPFDefinitionError(
-                        'CPF <{}> is missing a valid definition.'.format(name))
+                        f'CPF <{name}> is missing a valid definition.')
     
                 for g in grounded:
                     grounded_cpf = self._ground_single_cpf(
@@ -249,9 +292,13 @@ class RDDLGrounder(Grounder):
                     self.statesranges[g] = pvariable.range
                     self.nextstates[g] = next_state
                     self.prevstates[next_state] = g
-                    self.cpfs[next_state] = grounded_cpf.expr
+                    self.cpfs[next_state] = ([], grounded_cpf.expr)
                     self.cpforder[0].append(g)
-              
+                    self.gvar_to_cpforder[g] = 0
+                    self.gvar_to_type[g] = vtype
+                    self.gvar_to_pvar[next_state] = name
+                    self.gvar_to_type[next_state] = vtype
+                    
             elif pvariable.fluent_type == 'derived-fluent':
                 cpf = None
                 for cpfs in self.AST.domain.derived_cpfs:
@@ -260,13 +307,14 @@ class RDDLGrounder(Grounder):
                         break
                 if cpf is None:
                     raise RDDLMissingCPFDefinitionError(
-                        'CPF <{}> is missing a valid definition.'.format(name))
+                        f'CPF <{name}> is missing a valid definition.')
+                    
                 for g in grounded:
                     grounded_cpf = self._ground_single_cpf(
                         cpf, g, grounded_name_to_params_dict[g])
                     all_grounded_derived_cpfs.append(grounded_cpf)
                     self.derived[g] = pvariable.default
-                    self.cpfs[g] = grounded_cpf.expr
+                    self.cpfs[g] = ([], grounded_cpf.expr)
                     level = pvariable.level
                     if level is None:
                         level = 1
@@ -274,6 +322,8 @@ class RDDLGrounder(Grounder):
                         self.cpforder[level].append(g)
                     else:
                         self.cpforder[level] = [g]
+                    self.gvar_to_cpforder[g] = level
+                    self.gvar_to_type[g] = vtype
     
             elif pvariable.fluent_type == 'interm-fluent':
                 cpf = None
@@ -283,13 +333,14 @@ class RDDLGrounder(Grounder):
                         break
                 if cpf is None:
                     raise RDDLMissingCPFDefinitionError(
-                        'CPF <{}> is missing a valid definition.'.format(name))
+                        f'CPF <{name}> is missing a valid definition.')
+                    
                 for g in grounded:
                     grounded_cpf = self._ground_single_cpf(
                         cpf, g, grounded_name_to_params_dict[g])
                     all_grounded_interim_cpfs.append(grounded_cpf)
                     self.interm[g] = pvariable.default
-                    self.cpfs[g] = grounded_cpf.expr
+                    self.cpfs[g] = ([], grounded_cpf.expr)
                     level = pvariable.level
                     if level is None:
                         level = 1
@@ -297,7 +348,9 @@ class RDDLGrounder(Grounder):
                         self.cpforder[level].append(g)
                     else:
                         self.cpforder[level] = [g]
-
+                    self.gvar_to_type[g] = vtype
+                    self.gvar_to_cpforder[g] = level
+                    
             elif pvariable.fluent_type == 'observ-fluent':
                 cpf = None
                 for cpfs in self.AST.domain.observation_cpfs:
@@ -306,15 +359,18 @@ class RDDLGrounder(Grounder):
                         break
                 if cpf is None:
                     raise RDDLMissingCPFDefinitionError(
-                        'CPF <{}> is missing a valid definition.'.format(name))
+                        f'CPF <{name}> is missing a valid definition.')
+                    
                 for g in grounded:
                     grounded_cpf = self._ground_single_cpf(
                         cpf, g, grounded_name_to_params_dict[g])
                     all_grounded_observ_cpfs.append(grounded_cpf)
                     self.observ[g] = pvariable.default
                     self.observranges[g] = pvariable.range
-                    self.cpfs[g] = grounded_cpf.expr
+                    self.cpfs[g] = ([], grounded_cpf.expr)
                     self.cpforder[0].append(g)
+                    self.gvar_to_type[g] = vtype
+                    self.gvar_to_cpforder[g] = 0
 
         # self.AST.domain.cpfs = (self.AST.domain.cpfs[0], all_grounded_state_cpfs
         #                        )  # replacing the previous lifted entries
@@ -330,15 +386,16 @@ class RDDLGrounder(Grounder):
             return new_cpf
         if len(args) != len(variable_args):
             raise RDDLInvalidNumberOfArgumentsError(
-                f'Ground instance <{variable}> is of arity {len(variable_args)} but '
-                f'was expected to be of arity {len(args)} according to declaration.')
+                f'Ground instance <{variable}> is of arity {len(variable_args)}, '
+                f'but expected to be of arity {len(args)} according to definition.')
         args_dic = dict(zip(args, variable_args))
             
         # Parse cpf w.r.t cpf args and variables.
         # Fix name.
         cpf_base_name = new_cpf.pvar[1][0]
         cpf_name_grounding_variation = [[args_dic[arg] for arg in new_cpf.pvar[1][1]]]  # must be a nested list for func call
-        new_name = self._generate_grounded_names(cpf_base_name, cpf_name_grounding_variation)
+        new_name = self._generate_grounded_names(
+            cpf_base_name, cpf_name_grounding_variation)
         new_pvar = ('pvar_expr', (new_name, None))
         new_cpf.pvar = new_pvar
         new_cpf.expr = self._scan_expr_tree(new_cpf.expr, args_dic)
@@ -357,35 +414,33 @@ class RDDLGrounder(Grounder):
         """
 
         new_children = []
-        for instance_idx in range(len(instances_list)):
+        for instance in instances_list:
             updated_dict = copy.deepcopy(original_dict)
-            updated_dict.update(dict(zip(new_variables_list, instances_list[instance_idx])))
+            updated_dict.update(zip(new_variables_list, instance))
             new_children.append(self._scan_expr_tree(expression, updated_dict))
-        #--end for loop through instances
         new_expr = Expression((operation_string, tuple(new_children)))
         return new_expr
 
-
-
-
     def _scan_expr_tree_pvar(self, expr: Expression, dic) -> Expression:
         """Ground out a pvar expression."""
         if expr.args[1] is None:
-            # should really be etype = constant in parsed tree.
-            # This is a constant.
+            # This is a constant: should really be etype = constant in parsed tree.
             pass
+        
         elif expr.args[1]:
             variation_list = []
             for arg in expr.args[1]:
                 if arg not in dic:
                     raise RDDLUndefinedVariableError(
-                        'Parameter <{}> is not defined in call to <{}>.'.format(arg, expr.args[0]))
+                        f'Parameter <{arg}> is not defined in call to <{expr.args[0]}>.')
                 variation_list.append(dic[arg])
             variation_list = [variation_list]
             new_name = self._generate_grounded_names(expr.args[0], variation_list)[0]
             expr = Expression(('pvar_expr', (new_name, None)))
+            
         else:
-            raise RDDLInvalidExpressionError(f'Malformed expression <{str(expr)}>.')
+            raise RDDLInvalidExpressionError(f'Malformed expression <{expr}>.')
+        
         return expr
 
     def _scan_expr_tree_abr(self, expr, dic):
@@ -400,7 +455,8 @@ class RDDLGrounder(Grounder):
             self._scan_expr_tree(expr.args[1], dic),
             self._scan_expr_tree(expr.args[2], dic)
         ]
-        # TODO: add default case when no "else". For now, we are safe, as else is expected in rddl
+        # TODO: add default case when no "else". 
+        # For now, we are safe, as else is expected in rddl
         return Expression(('if', tuple(children_list)))
 
     def _scan_expr_tree_func(self, expr, dic):
@@ -418,8 +474,10 @@ class RDDLGrounder(Grounder):
         # and give feedback to the language writer for debugging.
         aggreg_type = expr.etype[1]
         if aggreg_type in AGGREG_OP_TO_STRING_DICT:
+            
             # Determine what the recursive op is for the aggreg type. Eg: sum = "+".
             aggreg_recursive_operation_string = AGGREG_OP_TO_STRING_DICT[aggreg_type]
+            
             # TODO: only for average operation, we need to first "/ n " for all others
             # we need to decide the recursive operation symbol "+" or "*" and iterate.
             # First let's collect the instances like (?x,?y) = (x1, y3) that satisfy,
@@ -429,7 +487,9 @@ class RDDLGrounder(Grounder):
             for arg in expr.args:
                 if arg[0] == 'typed_var':
                     instances_def_args += list(arg[1])
-            if instances_def_args:  # Then we iterate over the objects specified.
+            if instances_def_args:  
+                
+                # Then we iterate over the objects specified.
                 # All even indexes (incl 0) are variable names,
                 # all odd indexes are object types.
                 var_key_strings_list = [
@@ -445,15 +505,16 @@ class RDDLGrounder(Grounder):
                 ]
                 for var_idx in range(1, len(var_key_strings_list)):
                     instance_tuples = [
-                        tuple(list(instance_tuples[i]) + [self.objects[object_type_list[var_idx]][j]])
-                        for i in range(len(instance_tuples))
-                        for j in range(len(self.objects[object_type_list[var_idx]]))
+                        tuple(list(instance) + [objects])
+                        for instance in instance_tuples
+                        for objects in self.objects[object_type_list[var_idx]]
                     ]
                 object_instances_list = instance_tuples
                 expr = self.do_aggregate_expression_grounding(
                     dic, var_key_strings_list, object_instances_list,
                     aggreg_recursive_operation_string, arg
                 )  # Last arg is the expression with which the aggregation is done.
+                
                 if aggreg_type == 'avg':
                     num_instances = len(instance_tuples)  # Needed if this is an "Avg" operation.
                     # Then the 'expr' becomes lhs argument and
@@ -506,8 +567,9 @@ class RDDLGrounder(Grounder):
         if hasattr(self.AST.domain, 'constraints'):
             if self.AST.domain.constraints:
                 warnings.warn(
-                    'State-action constraints are not implemented in this RDDL version and will be ignored.',
-                    FutureWarning, stacklevel=2)
+                    f'State-action constraints are not implemented '
+                    f'in this RDDL version and will be ignored.',
+                    stacklevel=2)
     
         if hasattr(self.AST.domain, 'invariants'):
             for inv in self.AST.domain.invariants:
@@ -524,6 +586,35 @@ class RDDLGrounder(Grounder):
                     self.initstate[key] = val
                 else:
                     warnings.warn(
-                        'Init-state block initializes an undefined state fluent <{}>.'.format(key),
-                        FutureWarning, stacklevel=2)
-
+                        f'Init-state block initializes undefined state-fluent <{key}>.',
+                        stacklevel=2)
+                    
+    # added on Dec 17 (Mike)
+    def _extract_variable_information(self):
+        var_params, var_types, var_ranges = {}, {}, {}
+        for pvar in self.AST.domain.pvariables:
+            objects = pvar.param_types
+            if objects is None:
+                objects = []
+            if objects:
+                variations = list(self._ground_objects(objects))
+                grounded = self._generate_grounded_names(pvar.name, variations)
+            else:
+                grounded = [pvar.name]
+            for name in grounded:
+                if name in var_params:
+                    raise RDDLRepeatedVariableError(
+                        f'{pvar.fluent_type} <{pvar.name}> has the same name as '
+                        f'another {var_types[name]}.')
+                primed_name = (name + PRIME) if pvar.is_state_fluent() else name
+                var_params[name] = var_params[primed_name] = []        
+                var_types[name] = pvar.fluent_type
+                if pvar.is_state_fluent():
+                    var_types[primed_name] = 'next-state-fluent'                
+                var_ranges[name] = var_ranges[primed_name] = pvar.range 
+        grounded_names = {name: [name] for name in var_params}
+          
+        self.param_types, self.variable_types, self.variable_ranges = \
+            var_params, var_types, var_ranges
+        self.grounded_names = grounded_names
+        
