@@ -1,21 +1,21 @@
-from typing import Optional
-
 import matplotlib.pyplot as plt
+import matplotlib.patches as mpatches
 import numpy as np
 from PIL import Image
-
-from pyRDDLGym.Visualizer.StateViz import StateViz
-from pyRDDLGym.Core.Grounder.RDDLModel import RDDLModel
-import matplotlib.patches as mpatches
-
-from pyRDDLGym import Visualizer
-
 import sys
+from typing import Optional
+
+from pyRDDLGym.Core.Compiler.RDDLModel import PlanningModel
+from pyRDDLGym import Visualizer
+from pyRDDLGym.Visualizer.StateViz import StateViz
+
 
 class ReservoirVisualizer(StateViz):
-    def __init__(self, model: RDDLModel, grid_size: Optional[int] = [50,50], resolution: Optional[int] = [500,500]) -> None:
 
-        self._model= model
+    def __init__(self, model: PlanningModel,
+                 grid_size: Optional[int]=[50, 50],
+                 resolution: Optional[int]=[500, 500]) -> None:
+        self._model = model
         self._states = model.states
         self._nonfluents = model.nonfluents
         self._objects = model.objects
@@ -34,22 +34,20 @@ class ReservoirVisualizer(StateViz):
 
         # self.render()
         self.render()
-
     
     def build_object_layout(self) -> dict:
         
-        max_res_cap = {o:None for o in self._objects['res']}
-        upper_bound = {o:None for o in self._objects['res']}
-        lower_bound = {o:None for o in self._objects['res']}
-        rain_shape = {o:None for o in self._objects['res']}
-        rain_scale = {o:None for o in self._objects['res']}
-        downstream = {o:[] for o in self._objects['res']}
-        sink_res = {o:None for o in self._objects['res']}
-
-        rlevel = {o:None for o in self._objects['res']}
+        max_res_cap = {o: None for o in self._objects['res']}
+        upper_bound = {o: None for o in self._objects['res']}
+        lower_bound = {o: None for o in self._objects['res']}
+        rain_shape = {o: None for o in self._objects['res']}
+        rain_scale = {o: None for o in self._objects['res']}
+        downstream = {o: [] for o in self._objects['res']}
+        sink_res = {o: None for o in self._objects['res']}
+        rlevel = {o: None for o in self._objects['res']}
 
         # add none-fluents
-        for k,v in self._nonfluents.items():
+        for k, v in self._nonfluents.items():
             if 'RAIN_SHAPE_' in k:
                 point = k.split('_')[2]
                 rain_shape[point] = v
@@ -100,16 +98,19 @@ class ReservoirVisualizer(StateViz):
             if sink_res[o] == None:
                 sink_res[o] = False
 
-        object_layout = {'rain_shape': rain_shape, 'rain_scale': rain_scale, 'max_res_cap':max_res_cap, 'rlevel':rlevel,
-                         'upper_bound':upper_bound, 'lower_bound':lower_bound, 'downstream':downstream, 'sink_res':sink_res
+        object_layout = {'rain_shape': rain_shape, 'rain_scale': rain_scale,
+                         'max_res_cap': max_res_cap, 'rlevel': rlevel,
+                         'upper_bound': upper_bound, 'lower_bound': lower_bound,
+                         'downstream': downstream, 'sink_res': sink_res
                          }
         return object_layout
-    
 
     def init_canvas_info(self):
         interval = self._interval
         objects_set = set(self._objects['res'])
-        sink_res_set = set([k for k, v in self._object_layout['sink_res'].items() if v == True])
+        sink_res_set = set([k 
+                            for k, v in self._object_layout['sink_res'].items() 
+                            if v == True])
 
         all_child_set = []
         for i in self._object_layout['downstream'].values():
@@ -137,22 +138,18 @@ class ReservoirVisualizer(StateViz):
         row_num = len(level_list)
         col_num = max(len(i) for i in level_list)
 
-        canvas_size = (col_num*interval, row_num*interval)
+        canvas_size = (col_num * interval, row_num * interval)
         init_points = {}
         for i in range(len(level_list)):
             for j in range(len(level_list[i])):
-                init_x = 0 + interval*j
-                init_y = canvas_size[1] - interval*(i+1)    
+                init_x = 0 + interval * j
+                init_y = canvas_size[1] - interval * (i + 1)    
                 init_points[level_list[i][j]] = (init_x, init_y)
         conn_points = {}
         
         canvas_info = {'canvas_size':canvas_size, 'init_points':init_points}
 
-
         return canvas_info
-        
-
-
 
         # print(level_list)
         # print(self._object_layout['downstream'])
@@ -160,13 +157,10 @@ class ReservoirVisualizer(StateViz):
         # print(all_child_set)
         # print(all_root_set)
 
-        
-
         # node_links = {}
         # for o in self._objects['res']:
         #     for k,v in self._object_layout['downstream'].items():
 
-
         # plt.rcParams['axes.facecolor']='white'
         # fig_size = (num_row * 12, num_col*12)
         # fig = plt.figure(fig_size)
@@ -182,30 +176,31 @@ class ReservoirVisualizer(StateViz):
         ax = self._ax
         downstream = self._object_layout['downstream']
         init_points = self._canvas_info['init_points']
-        interval = self._interval*2/3
+        interval = self._interval * 2 / 3
 
         print(downstream)
         print(init_points)
 
-        for k,v in downstream.items():
-            top_point = (init_points[k][0] + interval/2, init_points[k][1])
+        for k, v in downstream.items():
+            top_point = (init_points[k][0] + interval / 2, init_points[k][1])
             for p in v:
-                bot_point = (init_points[p][0] + interval/2, init_points[p][1] + interval*1.25)
-                style = mpatches.ArrowStyle('Fancy', head_length=2, head_width=2, tail_width=0.01)
-                arrow = mpatches.FancyArrowPatch(top_point, bot_point, arrowstyle=style, color='k')
+                bot_point = (init_points[p][0] + interval / 2,
+                             init_points[p][1] + interval * 1.25)
+                style = mpatches.ArrowStyle('Fancy',
+                                            head_length=2, head_width=2,
+                                            tail_width=0.01)
+                arrow = mpatches.FancyArrowPatch(top_point, bot_point,
+                                                 arrowstyle=style, color='k')
                 ax.add_patch(arrow)
-
                 
         plt.show()
         sys.exit()
 
-
-
-    def render_res(self, res : str) -> tuple:
+    def render_res(self, res: str) -> tuple:
 
         fig = self._fig
         ax = self._ax
-        interval = self._interval*2/3
+        interval = self._interval * 2 / 3
         curr_t = res
         init_x, init_y = self._canvas_info['init_points'][curr_t]
 
@@ -221,8 +216,6 @@ class ReservoirVisualizer(StateViz):
         # plt.axis('scaled')
         # plt.axis('off')
 
-
-
         rlevel = self._object_layout['rlevel'][curr_t]
         rain_scale = self._object_layout['rain_scale'][curr_t]
         rain_shape = self._object_layout['rain_shape'][curr_t]
@@ -234,32 +227,48 @@ class ReservoirVisualizer(StateViz):
         sink_res = self._object_layout['sink_res'][curr_t]
         # downstream = self._object_layout['downstream'][curr_t]
 
-        maxL = [init_x, max_res_cap/100+init_y]
-        maxR = [init_x+interval, max_res_cap/100+init_y]
-        upL = [init_x, upper_bound/100+init_y]
-        upR = [init_x+interval, upper_bound/100+init_y]
-        lowL = [init_x, lower_bound/100+init_y]
-        lowR = [init_x+interval, lower_bound/100+init_y]
-
-        line_max = plt.Line2D((maxL[0], maxR[0]),(maxL[1], maxR[1]), ls='--', color='black',lw=1)
-        line_up = plt.Line2D((upL[0], upR[0]),(upL[1], upR[1]), ls='--', color='orange',lw=1)
-        line_low = plt.Line2D((lowL[0], lowR[0]),(lowL[1], lowR[1]), ls='--', color='orange',lw=1)
-        lineL = plt.Line2D((init_x, init_x), (init_y, init_y+interval), color='black',lw=1)
-        lineR = plt.Line2D((init_x+interval, init_x+interval), (init_y, init_y+interval), color='black',lw=1)
-        lineB = plt.Line2D((init_x, init_x+interval), (init_y, init_y), color='black',lw=1)
-
-        
-
-
-
-
-        water_rect = plt.Rectangle((init_x, init_y), interval, rlevel/100, fc='royalblue')
-        res_rect = plt.Rectangle((init_x, init_y + rlevel/100), interval, interval-rlevel/100, fc='lightgrey', alpha=0.5)
-        scale_rect = plt.Rectangle((init_x, init_y + interval), interval/2, interval/4, fc='deepskyblue', alpha=rain_scale/100)
-        shape_rect = plt.Rectangle((init_x + interval/2,init_y + interval), interval/2, interval/4, fc='darkcyan', alpha=rain_shape/100)
-
+        maxL = [init_x, max_res_cap / 100 + init_y]
+        maxR = [init_x + interval, max_res_cap / 100 + init_y]
+        upL = [init_x, upper_bound / 100 + init_y]
+        upR = [init_x + interval, upper_bound / 100 + init_y]
+        lowL = [init_x, lower_bound / 100 + init_y]
+        lowR = [init_x + interval, lower_bound / 100 + init_y]
 
+        line_max = plt.Line2D((maxL[0], maxR[0]),
+                              (maxL[1], maxR[1]),
+                              ls='--', color='black', lw=1)
+        line_up = plt.Line2D((upL[0], upR[0]),
+                             (upL[1], upR[1]),
+                             ls='--', color='orange', lw=1)
+        line_low = plt.Line2D((lowL[0], lowR[0]),
+                              (lowL[1], lowR[1]),
+                              ls='--', color='orange', lw=1)
+        lineL = plt.Line2D((init_x, init_x),
+                           (init_y, init_y + interval),
+                           color='black', lw=1)
+        lineR = plt.Line2D((init_x + interval, init_x + interval),
+                           (init_y, init_y + interval),
+                           color='black', lw=1)
+        lineB = plt.Line2D((init_x, init_x + interval),
+                           (init_y, init_y),
+                           color='black', lw=1)
 
+        water_rect = plt.Rectangle((init_x, init_y),
+                                   interval,
+                                   rlevel / 100,
+                                   fc='royalblue')
+        res_rect = plt.Rectangle((init_x, init_y + rlevel / 100),
+                                 interval,
+                                 interval - rlevel / 100,
+                                 fc='lightgrey', alpha=0.5)
+        scale_rect = plt.Rectangle((init_x, init_y + interval),
+                                   interval / 2,
+                                   interval / 4,
+                                   fc='deepskyblue', alpha=rain_scale / 100)
+        shape_rect = plt.Rectangle((init_x + interval / 2, init_y + interval),
+                                   interval / 2,
+                                   interval / 4,
+                                   fc='darkcyan', alpha=rain_shape / 100)
 
         ax.add_line(line_max)
         ax.add_line(line_up)
@@ -272,21 +281,29 @@ class ReservoirVisualizer(StateViz):
         ax.add_patch(res_rect)
         ax.add_patch(scale_rect)
         ax.add_patch(shape_rect)
-        
 
-
-
-
-
-        lineU = plt.Line2D((init_x + interval*1.25, init_x + interval*1.25), (init_y, init_y+interval), color='black',lw=1)
-        lineD = plt.Line2D((init_x + interval, init_x + interval*1.25), (init_y, init_y), color='black',lw=1)
+        lineU = plt.Line2D((init_x + interval * 1.25, init_x + interval * 1.25),
+                           (init_y, init_y + interval),
+                           color='black', lw=1)
+        lineD = plt.Line2D((init_x + interval, init_x + interval * 1.25),
+                           (init_y, init_y),
+                           color='black', lw=1)
         # conn_shape = plt.Rectangle((init_x + interval, init_y), interval/4, interval, fc='royalblue')
         if sink_res:
-            land_shape = plt.Rectangle((init_x + interval, init_y), interval/4, interval, fc='royalblue')
+            land_shape = plt.Rectangle((init_x + interval, init_y),
+                                       interval / 4,
+                                       interval,
+                                       fc='royalblue')
         else:
-            land_shape = plt.Rectangle((init_x + interval, init_y), interval/4, interval, fc='darkgoldenrod')
+            land_shape = plt.Rectangle((init_x + interval, init_y),
+                                       interval / 4,
+                                       interval,
+                                       fc='darkgoldenrod')
 
-        plt.text(init_x+interval*1.1, init_y+interval*1.1, "%s" % curr_t, color='black', fontsize = 5)
+        plt.text(init_x + interval * 1.1,
+                 init_y + interval * 1.1,
+                 "%s" % curr_t,
+                 color='black', fontsize=5)
 
         ax.add_patch(water_rect)
         ax.add_patch(res_rect)
@@ -329,27 +346,24 @@ class ReservoirVisualizer(StateViz):
         data = np.frombuffer(fig.canvas.tostring_rgb(), dtype=np.uint8)
         data = data.reshape(fig.canvas.get_width_height()[::-1] + (3,))
         return data
-        
 
-    def render(self, display:bool = True) -> np.ndarray:
+    def render(self, display:bool=True) -> np.ndarray:
         self._object_layout = self.build_object_layout()
         self._canvas_info = self.init_canvas_info()
         canvas_size = self._canvas_info['canvas_size']
 
-        self._fig = plt.figure(figsize = (canvas_size[0]/10, canvas_size[1]/10), dpi=500)
+        self._fig = plt.figure(figsize=(canvas_size[0] / 10, canvas_size[1] / 10),
+                               dpi=500)
         self._ax = plt.gca()
 
-        plt.xlim([0,canvas_size[0]])
-        plt.ylim([0,canvas_size[1]])
+        plt.xlim([0, canvas_size[0]])
+        plt.ylim([0, canvas_size[1]])
         plt.axis('scaled')
         plt.axis('off')
 
-
         # plt.show()
         # sys.exit()
 
-
-
         # layout_data = []
         for res in self._objects['res']:
             curr_t = res
@@ -368,20 +382,11 @@ class ReservoirVisualizer(StateViz):
         plt.pause(20)
         plt.close()
 
-
-
-        
-
         # fig1, ax1 = self.render_single_fig(curr_t, render_text=True)
 
-        
-
         # plt.show()
 
-
-
     # def build_res_shape(self, fig, ax, max_res_cap, upper_bound, lower_bound):
-        
 
     #     line_max = plt.Line2D((maxL[0], maxR[0]),(maxL[1], maxR[1]), ls='--', color='black',lw=3)
     #     line_up = plt.Line2D((upL[0], upR[0]),(upL[1], upR[1]), ls='--', color='orange',lw=3)
@@ -403,8 +408,6 @@ class ReservoirVisualizer(StateViz):
     #     ax.add_line(lineR)
     #     ax.add_line(lineB)
         
-
-        
     # def build_res_fill(self, fig, ax, rlevel, rain_scale, rain_shape):
 
     #     water_rect = plt.Rectangle((0,0), 10, rlevel/100, fc='royalblue')
@@ -437,29 +440,19 @@ class ReservoirVisualizer(StateViz):
 
     #     ax.add_patch(land_shape)
     #     plt.text(10.1, 2.1, "Sink: %s" % sink_res, color='black', fontsize = 15)
-       
 
-    def display_img(self, duration:float = 0.5) -> None:
+    def display_img(self, duration:float=0.5) -> None:
 
         plt.imshow(self._data, interpolation='nearest')
         plt.axis('off')
         plt.show(block=False)
         plt.pause(duration)
         plt.close()
-    
 
-    def save_img(self, path:str ='./pict.png') -> None:
+    def save_img(self, path:str='./pict.png') -> None:
         
         im = Image.fromarray(self._data)
         im.save(path)
-        
-
-
-
-
-   
-  
-        
 
     # def render(self, display:bool = True) -> np.ndarray:
 
@@ -510,13 +503,9 @@ class ReservoirVisualizer(StateViz):
     #     plt.show(block=False)
     #     plt.pause(duration)
     #     plt.close()
-    
 
     # def save_img(self, path:str ='./pict.png') -> None:
         
     #     im = Image.fromarray(self._data)
     #     im.save(path)
-        
-
-    
 
